#!/bin/bash
is_quiet=0
is_full=1
set_full=0
is_collectdump=0
is_clientdump=0

hostInfoPath=${hostInfoPath:="`pwd`"}
hostinfo_tarName="output.tar.gz"

readonly resultPath="${hostInfoPath}/result"
readonly hbaInfoFile="${resultPath}/hba.txt"
readonly upInfoFile="${resultPath}/up.txt"
readonly iscsiInfoFile="${resultPath}/iscsi.txt"
readonly scsiInfoFile="${resultPath}/scsi.txt"
readonly hostInfoFile="${resultPath}/host.txt"
readonly historyInfoFile="${resultPath}/history.txt"
readonly srvInfoFile="${resultPath}/service.txt"
readonly dmesgFile="${resultPath}/dmesg.txt"
readonly networkFile="${resultPath}/network.txt"
readonly udevFile="${resultPath}/udev.txt"
readonly lvmFile="${resultPath}/lvm.txt"
readonly dmFile="${resultPath}/dm.txt"
readonly printkFile="${resultPath}/printk.txt"
readonly errorFile="${resultPath}/error_report"

g10s_OSVersion=""
g10s_OSReleaseFile=""

lockFile="/tmp/HostinfoAnalyzeTool-linux.lock"

readonly RUN_CONFLICT_EXITCODE=200
readonly NO_ENOUGH_SPACE_EXITCODE=201
readonly FILE_DIR_NOT_EXIST=202

CheckSystemType()
{
    OS_TYPE=`uname`
    if [ "${OS_TYPE}" != "Linux" ]
    then
        echo "Only support Linux! The OS is ${OS_TYPE}"
        exit 1
    fi
}

function host_exit()
{
    unlockShell "${lockFile}"
    exit "$1"
}

function print_error()
{
   echo -e -n "\033[31m[ERROR]\033[0m ";echo "$1"
   return 0
}

function g10s_curUnamer()
{
    echo "`uname -r 2>/dev/null`"
}

function lockShell()
{
    local thepid=""
    local ret=""

    if [ "$1" = "" ]
    then
        print_error "need lock file param"
        exit 1
    fi

    if [ -f "${1}" ]
    then
        thepid=`cat "${1}"`
        if [ "${thepid}" != "" ]
        then
            ret=`ps ax |awk '{print $1}'| grep -c "^[[:blank:]]*${thepid}[[:blank:]]*$"`
            if [ ${ret} -eq 1 ]
            then
                print_error "this shell is already running, exit."
                exit ${RUN_CONFLICT_EXITCODE}
            fi
        fi
    fi

    echo $$ > "${1}"
    chmod go+w "${1}"
    return 0
}

function unlockShell()
{
    if [ "$1" = "" ]
    then
        return 1
    fi

    if [ -f "${1}" ]
    then
        rm -f "${1}"
    fi

    return 0
}

function print_event()
{
    if [ -z "$1" ] || [ -z "$2" ]
    then
        echo "ERROR:Param Error."
        host_exit 1
    fi

    echo "`hostname`:`pwd`# $1" >> $2

    return 0
}

function useage()
{
    host_exit 0
}

function exec_cmd()
{
    if [ -z "$1" ] || [ -z "$2" ]
    then
        print_error "param error."
        host_exit 1
    fi

    if [ ! -f "$2" ]
    then
        touch "$2"
        date >> $2
    fi

    print_event "$1" "$2"
    eval "$1 >>$2 2>>${errorFile}"
    echo "" >>"$2"

    return 0
}

function copy_file()
{
    if [ "$1" = "" ] || [ "$2" = "" ]
    then
        print_error "param error."
        host_exit 1
    fi

    cp -rf "$1" "$2" 2>>${errorFile}
}

function get_history_info()
{
    history -w 2>/dev/null
    if [ -f /root/.bash_history ]
    then
        exec_cmd "cat /root/.bash_history" "${historyInfoFile}"
    else
        exec_cmd "history" "${historyInfoFile}"
    fi

    return 0
}

function __g10s_getOSForSuSE()
{
    g10s_OSVersion=""
    local thisKernel=`uname -r 2>/dev/null`
    local vernum=`cat /etc/SuSE-release | sed -n 's/^[[:blank:]]*VERSION[[:blank:]]*=[[:blank:]]*\([0-9][0-9]*\)[[:blank:]]*$/\1/p'`
    local verSP=`cat /etc/SuSE-release | sed -n 's/^[[:blank:]]*PATCHLEVEL[[:blank:]]*=[[:blank:]]*\([0-9][0-9]*\)[[:blank:]]*$/\1/p'`
    if [ "${verSP}" = "" ]
    then
        verSP=0
    fi
    if [ "${vernum}" != "" ]
    then
        g10s_OSVersion="SLES${vernum}SP${verSP}"
    fi
    if [ "${verSP}" = "2" ] && [ "${vernum}" = "9" ] && [ `echo ${thisKernel} | grep -c "201"` -ne 0 ]
    then
        g10s_OSVersion="SLES${vernum}SP${verSP}_201"
    fi

    return 0
}

function __g10s_getOSForAsianux()
{
    g10s_OSVersion=""
    if [ `cat /etc/asianux-release | grep -c "Asianux Server 4"` -ne 0 ]
    then
        g10s_OSVersion="ASIANUX4"
    elif [ `cat /etc/asianux-release | grep -c "Asianux Server 3"` -ne 0 ]
    then
        g10s_OSVersion="ASIANUX3"
    elif [ `cat /etc/asianux-release | grep -c "Asianux release 2.0"` -ne 0 ]
    then
        g10s_OSVersion="ASIANUX2"
    else
        g10s_OSVersion="`cat /etc/asianux-release`"
    fi

    return 0
}

function __g10s_getOSForRedHat()
{
    g10s_OSVersion=""
    if [ `cat /etc/redhat-release | grep -c -E "(Red Hat Enterprise Linux Server release 7)"` -ne 0 ] && [  ! -f /etc/uvp_version ]
    then
        g10s_OSVersion="RHEL7"
    elif [ `cat /etc/redhat-release | grep -c -E "(CentOS release 7|CentOS Linux release 7)"` -ne 0 ] && [  ! -f /etc/uvp_version ]
    then
        g10s_OSVersion="CentOS7"
    elif [ `cat /etc/redhat-release | grep -c -E "(Red Hat Enterprise Linux Server release 6)"` -ne 0 ]
    then
        g10s_OSVersion="RHEL6"
    elif [ `cat /etc/redhat-release | grep -c -E "(CentOS release 6|CentOS Linux release 6)"` -ne 0 ]
    then
        g10s_OSVersion="CentOS6"
    elif [ `cat /etc/redhat-release | grep -c -E "(Red Hat Enterprise Linux Server release 5)"` -ne 0 ]
    then
        g10s_OSVersion="RHEL5"
    elif [ `cat /etc/redhat-release | grep -c -E "(CentOS release 5)"` -ne 0 ]
    then
        g10s_OSVersion="CentOS5"
    elif [ `cat /etc/redhat-release | grep -c -E "(Red Hat Enterprise Linux AS release 4)"` -ne 0 ]
    then
        g10s_OSVersion="RHEL4"
    elif [ `cat /etc/redhat-release | grep -c -E "(CentOS release 4)"` -ne 0 ]
    then
        g10s_OSVersion="CentOS4"
    elif [ `cat /etc/redhat-release | grep -c -E "(Kylin release 3.0|Kylin release V3)"` -ne 0 ]
    then
        g10s_OSVersion="KYLIN3SP0"
    elif [ `cat /etc/redhat-release | grep -c "NeoKylin Linux Server release 3.2"` -ne 0 ] && [ `cat /etc/redhat-release | grep -c "NeoKylin 3.2.1"` -eq 0 ] && [ `cat /etc/redhat-release | grep -c "NeoKylin 3.2.2"` -eq 0 ]
    then
        if [ `echo "${thisKernel}" | grep -c "2.6.32.358"` -ne 0 ]
        then
            g10s_OSVersion="KYLIN3SP24"
        else
            g10s_OSVersion="KYLIN3SP23"
        fi
    elif [ `cat /etc/redhat-release | grep -c "NeoKylin 3.2.1"` -ne 0 ]
    then
        g10s_OSVersion="KYLIN3SP21"
    elif [ `cat /etc/redhat-release | grep -c "NeoKylin 3.2.2"` -ne 0 ]
    then
        g10s_OSVersion="KYLIN3SP22"
    else
        g10s_OSVersion="`cat /etc/redhat-release`"
    fi
    return 0
}

function __g10s_getOSForUVP()
{
    local thisKernel=`uname -r 2>/dev/null`

    if [ -f /etc/UVP-release  ] && [ -f /etc/uvp_version ]
    then
        if [ `cat "/etc/uvp_version" | grep "uvp_version" | grep -c "V100R002"` -eq 1 ]
        then
            g10s_OSVersion="UVPV1R2"
        elif [ `cat "/etc/uvp_version" | grep "uvp_version" | grep -c "V100R003"` -eq 1 ]
        then
            g10s_OSVersion="UVPV1R3"
            thisKernel=$(echo "${thisKernel}")
            if [ $(echo "${thisKernel}" | grep -c "^3\.0\.93-0\.8-.*") -eq 1 ]
            then
                g10s_OSVersion="UVPV1R32"
            fi
        fi
    elif [ -f /etc/uvp_version ] && [ ! -f /etc/UVP-release ]
    then
        g10s_OSVersion="UVPV1R32"
    elif [ `cat /etc/issue | grep "SUSE" | grep -c "11 SP2"` -gt 0 ] && [ `rpm -qa | grep -c "UVP_Compile_Tool_V1R2"` -gt 0 ]
    then
        g10s_OSVersion="UVPV1R2"
    elif [ `cat /etc/issue | grep "SUSE" | grep -c "11 SP2"` -gt 0 ] && [ `rpm -qa | grep -c "UVP_Compile_Tool_V1R3"` -gt 0 ]
    then
        g10s_OSVersion="UVPV1R3"
    elif [ `cat /etc/issue | grep "SUSE" | grep -c "11 SP3"` -gt 0 ] && [ `rpm -qa | grep -c "UVP_Compile_Tool_V1R3"` -gt 0 ]
    then
        g10s_OSVersion="UVPV1R32"
    fi
    if [ -f /boot/grub/menu.lst ] && [ `cat /boot/grub/menu.lst | grep -c "^[[:blank:]]*title[[:blank:]].*UVP.*V100R001C03"` -ne 0 ]
    then
        g10s_OSVersion="UVPV1R1C03"
    fi
    return 0
}

function __g10s_getOSForOracleLinux()
{
    g10s_OSVersion=""
    if [ `cat /etc/oracle-release | grep -c "Oracle Linux Server release 6"` -ne 0 ]
    then
        uVersion=`cat /etc/oracle-release | awk -F "." '{print $2}'`
        if [ "${uVersion}" = "" ]
        then
            g10s_OSVersion="OL6U0"
        else
            g10s_OSVersion="OL6U${uVersion}"
        fi
    elif [ `cat /etc/oracle-release | grep -c "Oracle Linux Server release 5"` -ne 0 ]
    then
        uVersion=`cat /etc/oracle-release | awk -F "." '{print $2}'`
        if [ "${uVersion}" = "" ]
        then
            g10s_OSVersion="OL5U0"
        else
            g10s_OSVersion="OL5U${uVersion}"
        fi
    else
        g10s_OSVersion="`cat /etc/oracle-release`"
    fi
    return 0
}

function __g10s_getOSForROCKY4SP2()
{
    if [ `cat /etc/issue | grep "4.2.35" |wc -l` -ne 0 ]
    then
        g10s_OSVersion="ROCKY_`g10s_curUnamer | awk -F "-" '{print $1}'|sed 's/[[:blank:]]//g'`"
    elif [ `cat /etc/issue | grep -E "4.2.39" |wc -l` -ne 0 ]
    then
        g10s_OSVersion="ROCKY_4.2.39"
    elif [ `cat /etc/issue | grep -E "4.2.40" |wc -l` -ne 0 ]
    then
        g10s_OSVersion="ROCKY_4.2.40"
    else
        g10s_OSVersion="ROCKY `cat /etc/issue`"
    fi
    return 0
}

function get_os_name()
{
    g10s_OSVersion="NU"
    g10s_OSReleaseFile=""
    if [ -f /etc/SuSE-release ]
    then
        g10s_OSReleaseFile="/etc/SuSE-release"
        __g10s_getOSForSuSE
    elif [ -f /etc/asianux-release ]
    then
        g10s_OSReleaseFile="/etc/asianux-release"
        __g10s_getOSForAsianux
    elif [ -f /etc/oracle-release ]
    then
        g10s_OSReleaseFile="/etc/oracle-release"
        __g10s_getOSForOracleLinux
    elif [ -f /etc/fedora-release ]
    then
        g10s_OSReleaseFile="/etc/fedora-release"
        g10s_OSVersion="`cat /etc/fedora-release`"
    elif [ -f /etc/redhat-release ] && [ ! -f /etc/asianux-release ] && [ ! -f /etc/oracle-release ] && [ ! -f /etc/fedora-release ]
    then
        g10s_OSReleaseFile="/etc/redhat-release"
        __g10s_getOSForRedHat

    elif [ -f /etc/linx-release ] && [ ! -f /etc/debian_version ]
    then
        g10s_OSReleaseFile="/etc/linx-release"
        g10s_kernelType="normal"
        __g10s_getOSForROCKY4SP2
    elif [ -f /etc/deepin_version ] && [ -f /etc/debian_version ]
    then
        g10s_OSReleaseFile="/etc/deepin_version"
        g10s_kernelType=`g10s_curUnamer | awk -F "-" '{print $NF}'`
        g10s_OSVersion="DEEPIN`cat /etc/deepin-version | grep "Version" | awk -F "=" '{print $2}'`"
    elif [ -f "/etc/debian_version" ]
    then
        g10s_OSReleaseFile="/etc/debian_version"

        if [ `cat /etc/issue | grep "Linx" -c` -gt 0 ]
        then
            g10s_OSVersion="ROCKY`cat /etc/debian_version|awk -F "." '{print $1}'`SP`cat /etc/debian_version|awk -F "." '{print $3}'`"
        elif [ `cat /etc/issue | grep "Ubuntu" -c` -gt 0 ]
        then
            g10s_OSVersion="Ubuntu `cat /etc/lsb-release | grep DISTRIB_RELEASE | awk -F "=" '{print $2}'`"
        elif [ `cat /etc/issue | grep "Debian" -c` -gt 0 ]
        then
            g10s_OSVersion="Debian `cat /etc/debian_version`"
        else
            : 
        fi
    fi
    __g10s_getOSForUVP

    if [ "${g10s_OSVersion}" != "NU" ]
    then
        echo ${g10s_OSVersion} > ${resultPath}/osname
    else
        echo "Unknown, failed to recognise!" > ${resultPath}/osname
        if [ -f /etc/issue ]
        then
            echo "The /etc/issue information:"  >> ${resultPath}/osname
            cat /etc/issue >> ${resultPath}/osname
        fi
    fi
}

function get_os_info()
{
    mkdir -p "${resultPath}/etc/sysconfig/"
    mkdir -p "${resultPath}/etc/infiniband/"
    mkdir -p "${resultPath}/sbin/"
    mkdir -p "${resultPath}/etc/rc.d/"
    mkdir -p "${resultPath}/proc/"

    get_os_name
    for release in `ls /etc/*-release 2>/dev/null`
    do
        copy_file "${release}" "${resultPath}/etc/"
    done

    copy_file "/etc/hosts" "${resultPath}/proc/"
    copy_file "/etc/fstab" "${resultPath}/proc/"
    copy_file "/etc/debian_version" "${resultPath}/etc/"
    copy_file "/etc/modprobe.conf" "${resultPath}/etc/"
    copy_file "/etc/modprobe.conf.local" "${resultPath}/etc/"
    copy_file "/etc/modprobe.d/nxupmodules.conf" "${resultPath}/"
    copy_file "/etc/modprobe.d/mppmodules.conf" "${resultPath}/"
    copy_file "/etc/sysconfig/kernel" "${resultPath}/etc/sysconfig/"
    copy_file "/etc/infiniband/openib.conf" "${resultPath}/etc/infiniband/"
    copy_file "/etc/rc.d/rc.sysinit" "${resultPath}/etc/rc.d/"
    copy_file "/etc/up.hostinfo" "${resultPath}/etc/"
    copy_file "/etc/UltraPath.packages.info" "${resultPath}/etc/"
    copy_file "/etc/up.conf" "${resultPath}/etc/"
    copy_file "/etc/issue" "${resultPath}/etc/"
    copy_file "/etc/inittab" "${resultPath}/etc/"
    copy_file "/sbin/mkinitrd" "${resultPath}/sbin/"
    copy_file "/proc/nxup_lun_map" "${resultPath}/proc/"
    copy_file "/proc/nxup_pia" "${resultPath}/proc/"
    copy_file "/proc/nxup_lun_map_a" "${resultPath}/proc/"
    copy_file "/proc/nxup_lun_map_b" "${resultPath}/proc/"
    copy_file "/etc/filesystems" "${resultPath}/etc/"
    copy_file "/etc/user_cfg.xml" "${resultPath}/etc/"

    exec_cmd "date" "${hostInfoFile}"
    exec_cmd "date -u" "${hostInfoFile}"
    exec_cmd "mount" "${hostInfoFile}"
    exec_cmd "uname -a" "${hostInfoFile}"
    exec_cmd "ls -l /boot" "${hostInfoFile}"
    exec_cmd "ls -l /boot/grub" "${hostInfoFile}"
    exec_cmd "ls -l /opt" "${hostInfoFile}"
    exec_cmd "getconf LONG_BIT" "${hostInfoFile}"
    exec_cmd "ps aux" "${hostInfoFile}"
    exec_cmd "crontab -l" "${hostInfoFile}"
    exec_cmd "hostname" "${hostInfoFile}"
    exec_cmd "last reboot" "${hostInfoFile}"
    exec_cmd "rpm -qa" "${hostInfoFile}"
    exec_cmd "ls -l /lib/modules/`uname -r`/kernel/drivers/scsi/" "${hostInfoFile}"
    exec_cmd "lsmod" "${hostInfoFile}"
    exec_cmd "procinfo" "${hostInfoFile}"
    exec_cmd "ls -l /var/crash/" "${hostInfoFile}"
    exec_cmd "uptime" "${hostInfoFile}"
    exec_cmd "ulimit -a" "${hostInfoFile}"
    exec_cmd "hostid" "${hostInfoFile}"
    exec_cmd "env" "${hostInfoFile}"
    exec_cmd "echo $BASH" "${hostInfoFile}"
    exec_cmd "cat /proc/cpuinfo" "${hostInfoFile}"
    exec_cmd "echo cpunumber:" "${hostInfoFile}"
    exec_cmd "cat /proc/cpuinfo |grep processor|wc -l" "${hostInfoFile}"
    exec_cmd "cat /proc/slabinfo" "${hostInfoFile}"
    exec_cmd "cat /proc/meminfo" "${hostInfoFile}"
    exec_cmd "cat /proc/cmdline" "${hostInfoFile}"
    exec_cmd "dmidecode" "${hostInfoFile}"
    exec_cmd "cat /proc/sys/kernel/printk" "${printkFile}"
    exec_cmd "whoami" "${hostInfoFile}"
    exec_cmd "id" "${hostInfoFile}"
    exec_cmd "swapon -s" "${hostInfoFile}"
    exec_cmd "vmstat -s" "${hostInfoFile}"
    exec_cmd "free -m" "${hostInfoFile}"
    exec_cmd "free -s 1 -c 10" "${hostInfoFile}"

    return 0
}

function get_each_hba_info()
{
    local num=0
    FCHOSTDIR=`ls /sys/class/fc_host/ 2>/dev/null`
    for hh in ${FCHOSTDIR}; do
        let num=${num}+1
        FCDIR=/sys/class/fc_host/${hh}
        SCSIDIR=/sys/class/scsi_host/${hh}
    FILELIST=`ls /sys/class/scsi_host/${hh}`
        FILELIST2=`ls /sys/class/fc_host/${hh}`
    if [ -f ${SCSIDIR}/lpfc_link_speed -o -f ${SCSIDIR}/driver_name -o -f ${SCSIDIR}/version_product -o -f ${SCSIDIR}/model_name ]; then
        DRV=""; FW=""; TMO=""; FOUND_HBA=1;

        if [[ `echo ${FILELIST} | awk '$0 ~ /modeldesc/'` ]]; then
                MODDESC=`cat ${SCSIDIR}/modeldesc |awk '{print $0}'`
        elif  [[ `echo ${FILELIST} | awk '$0 ~ /model_desc /'` ]]; then
                MODDESC=`cat ${SCSIDIR}/model_desc |awk '{print $0}'`
        elif  [[ `echo ${FILELIST} | awk '$0 ~ /model_description/'` ]]; then
                MODDESC=`cat ${SCSIDIR}/model_description |awk '{print $0}'`
        fi

        if [[ `echo ${FILELIST} | awk '$0 ~ /modelname/'` ]]; then
                MODEL=`cat ${SCSIDIR}/modelname |awk '{print $0}'`
        elif [[ `echo ${FILELIST} | awk '$0 ~ /model_name/'` ]]; then
                MODEL=`cat ${SCSIDIR}/model_name |awk '{print $0}'`
        elif [[ `echo ${FILELIST} | awk '$0 ~ /model /'` ]]; then
                MODEL=`cat ${SCSIDIR}/model |awk '{print $0}'`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /serialnum/'` ]]; then
                SERNUM=`cat ${SCSIDIR}/serialnum |awk '{print $0}'`
        elif [[ `echo ${FILELIST} |awk '$0 ~ /serial_num /'` ]]; then
                SERNUM=`cat ${SCSIDIR}/serial_num |awk '{print $0}'`
        elif [[ `echo ${FILELIST} |awk '$0 ~ /serial_number/'` ]]; then
                SERNUM=`cat ${SCSIDIR}/serial_number |awk '{print $0}'`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /driver_name/'` ]]; then
                DRV_NAME=`cat ${SCSIDIR}/driver_name |awk '{print $0}'`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /lpfc_drvr_version/'` ]]; then
                DRV_VER=`cat ${SCSIDIR}/lpfc_drvr_version |awk '{print $7}'`; LPFC=1
        elif [[ `echo ${FILELIST} |awk '$0 ~ /driver_version/'` ]]; then
                DRV_VER=`cat ${SCSIDIR}/driver_version |awk '{print $0}'`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /fwrev/'` ]]; then
                FW_VER=`cat ${SCSIDIR}/fwrev |awk '{print $1}'`
        elif [[ `echo ${FILELIST} |awk '$0 ~ /fw_version/'` ]]; then
                FW_VER=`cat ${SCSIDIR}/fw_version |awk '{print $1}'`
        elif [[ `echo ${FILELIST} |awk '$0 ~ /firmware_version/'` ]]; then
                FW_VER=`cat ${SCSIDIR}/firmware_version |awk '{print $1}'`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /option_rom_version/'` ]]; then
                BIOS_VER=`cat ${SCSIDIR}/option_rom_version | awk '{print $0}'`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ /option_rom_version/'` ]]; then
                BIOS_VER=`cat ${FCDIR}/option_rom_version | awk '{print $0}'`
        elif [[ `echo ${FILELIST} |awk '$0 ~ /optrom_bios_version/'` ]]; then
                BIOS_VER=`cat ${SCSIDIR}/optrom_bios_version | awk '{print $0}'`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ /optrom_bios_version/'` ]]; then
                BIOS_VER=`cat ${FCDIR}/optrom_bios_version | awk '{print $0}'`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ / state /'` ]]; then
            HBA_STATE=`cat ${SCSIDIR}/state | sed '$!N;s/\n//' | sed -e 's/\s\s//g' | awk '{print $0}'`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ / state /'` ]]; then
            HBA_STATE=`cat ${FCDIR}/state | sed '$!N;s/\n//' | sed -e 's/\s\s//g' | awk '{print $0}'`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /lpfc_lun_queue_depth/'` ]]; then
            LQD=`cat ${SCSIDIR}/lpfc_lun_queue_depth | awk '{print $0}'`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ /lpfc_lun_queue_depth/'` ]]; then
            LQD=`cat ${FCDIR}/lpfc_lun_queue_depth | awk '{print $0}'`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /lpfc_nodev_tmo/'` ]]; then
            TMO=`cat ${SCSIDIR}/lpfc_nodev_tmo | sed -e 's/\n\r//g' | awk '{print $0}'`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ /lpfc_nodev_tmo/'` ]]; then
            TMO=`cat ${FCDIR}/lpfc_nodev_tmo | sed -e 's/\n\r//g' | awk '{print $0}'`
        elif [[ `echo ${FILELIST} |awk '$0 ~ /dev_loss_tmo/'` ]]; then
            TMO=`cat ${SCSIDIR}/dev_loss_tmo | sed -e 's/\n\r//g' | awk '{print $0}'`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ /dev_loss_tmo/'` ]]; then
            TMO=`cat ${FCDIR}/dev_loss_tmo | sed -e 's/\n\r//g' | awk '{print $0}'`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /node_name/'` ]]; then
            WWNN=`cat ${SCSIDIR}/node_name | awk '{print $0}'| cut -c3-`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ /node_name/'` ]]; then
            WWNN=`cat ${FCDIR}/node_name | awk '{print $0}'| cut -c3-`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /port_name/'` ]]; then
            WWPN=`cat ${SCSIDIR}/port_name | awk '{print $0}'| cut -c3-`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ /port_name/'` ]]; then
            WWPN=`cat ${FCDIR}/port_name | awk '{print $0}'| cut -c3-`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /fabric_name/'` ]]; then
                WWFN=`cat ${SCSIDIR}/fabric_name |awk '{print $0}' | cut -c3-`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ /fabric_name/'` ]]; then
                WWFN=`cat ${FCDIR}/fabric_name |awk '{print $0}' | cut -c3-`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /number_of_ports/'` ]]; then
            NUM_PORTS=`cat ${SCSIDIR}/number_of_ports | awk '{print $0}'`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ /number_of_ports/'` ]]; then
            NUM_PORTS=`cat ${FCDIR}/number_of_ports | awk '{print $0}'`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /port_type/'` ]]; then
            PORT_TYPE=`cat ${SCSIDIR}/port_type | awk '{print $0}'`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ /port_type/'` ]]; then
            PORT_TYPE=`cat ${FCDIR}/port_type | awk '{print $0}'`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /tgtid_bind_type/'` ]]; then
                TGTID_BIND_TYPE=`cat ${SCSIDIR}/tgtid_bind_type |awk '{print $0}'`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ /tgtid_bind_type/'` ]]; then
                TGTID_BIND_TYPE=`cat ${FCDIR}/tgtid_bind_type |awk '{print $0}'`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /symbolic_name/'` ]]; then
                SYMB_NAME=`cat ${SCSIDIR}/symbolic_name |awk '{print $0}'`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ /symbolic_name/'` ]]; then
                SYMB_NAME=`cat ${FCDIR}/symbolic_name |awk '{print $0}'`
        fi

        if [[ `echo ${FILELIST} |awk '$0 ~ /port_state/'` ]]; then
                PORT_STATE=`cat ${SCSIDIR}/port_state |awk '{print $0}'`
        elif [[ `echo ${FILELIST2} |awk '$0 ~ /port_state/'` ]]; then
                PORT_STATE=`cat ${FCDIR}/port_state |awk '{print $0}'`
        fi

        printf "information of HBA${num}:\n" >> ${hbaInfoFile}
        if [ -n "${MODDESC}" ]; then
        printf "Model Description: ${MODDESC}\n" >> ${hbaInfoFile}
        else
        printf "Model Description: Not found\n" >> ${hbaInfoFile}
        fi

        if [ -n "${MODEL}" ]; then
        printf "Model: ${MODEL}\n" >> ${hbaInfoFile}
        else
        printf "Model: Not found\n" >> ${hbaInfoFile}
        fi

        if [ -n "${SERNUM}" ]; then
                printf "Serial Number: ${SERNUM}\n" >> ${hbaInfoFile}
        else
                printf "Serial Number: Not found\n" >> ${hbaInfoFile}
        fi

        if [ -n "${DRV_NAME}" ]; then
                printf "Driver Name: ${DRV_NAME}\n" >> ${hbaInfoFile}
        else
                printf "Driver Name: Not found\n" >> ${hbaInfoFile}
        fi

        if [ -n "${DRV_VER}" ]; then
                printf "Driver Version: ${DRV_VER}\n" >> ${hbaInfoFile}
        else
                printf "Driver Version: Not found\n" >> ${hbaInfoFile}
        fi

        if [ -n "${FW_VER}" ]; then
                printf "Firmware Version: ${FW_VER}\n" >> ${hbaInfoFile}
        else
                printf "Firmware Version: Not found\n" >> ${hbaInfoFile}
        fi

            if [ -n "${BIOS_VER}" ]; then
        printf "BIOS Version: ${BIOS_VER}\n" >> ${hbaInfoFile}
        else
        printf "BIOS Version: Not found\n" >> ${hbaInfoFile}
        fi

            if [ -n "${HBA_STATE}" ]; then
                printf "HBA State: ${HBA_STATE}\n" >> ${hbaInfoFile}
        else
                printf "HBA State: Not found\n" >> ${hbaInfoFile}
        fi

            if [ -n "${WWNN}" ]; then
                printf "WW Node Name: ${WWNN}\n" >> ${hbaInfoFile}
        else
                printf "WW Node Name: Not found\n" >> ${hbaInfoFile}
        fi

            if [ -n "${WWPN}" ]; then
                printf "WW Port Name: ${WWPN}\n" >> ${hbaInfoFile}
        else
                printf "WW Port Name: Not found\n" >> ${hbaInfoFile}
        fi

            if [ -n "${WWFN}" ]; then
            printf "WW Fabric Name: ${WWFN}\n" >> ${hbaInfoFile}
        else
            printf "WW Fabric Name: Not found\n" >> ${hbaInfoFile}
        fi

            if [ -n "${PORT_TYPE}" ]; then
                printf "Port Type: ${PORT_TYPE}\n" >> ${hbaInfoFile}
        else
                printf "Port Type: Not found\n" >> ${hbaInfoFile}
        fi

            if [ -n "${TGTID_BIND_TYPE}" ]; then
            printf "TargetID Bind Type: ${TGTID_BIND_TYPE}\n" >> ${hbaInfoFile}
        else
            printf "TargetID Bind Type: Not found\n" >> ${hbaInfoFile}
        fi

            if [ -n "${TMO}" ]; then
                printf "Disk Timeout: ${TMO}\n" >> ${hbaInfoFile}
        else
                printf "Disk Timeout: Not found\n" >> ${hbaInfoFile}
        fi

            if [ -n "${LQD}" ]; then
                printf "Lun Queue Depth: ${LQD}\n" >> ${hbaInfoFile}
        else
                printf "Lun Queue Depth: Not found\n" >> ${hbaInfoFile}
        fi

        printf "end of HBA${num}\n\n" >> ${hbaInfoFile}
    fi
    done
}

function get_hba_info()
{
    get_each_hba_info
    [ -f /proc/mpt/version ] &&  exec_cmd "cat /proc/mpt/version" "${hbaInfoFile}"

    if [ -d /proc/mpt ]
    then
       exec_cmd "ls -l /proc/mpt" "${hbaInfoFile}"
       exec_cmd "cat /proc/mpt/*/info" "${hbaInfoFile}"
    fi

    [ `lsmod 2>/dev/null|grep -c "qla2xxx"` -ne 0 ] && exec_cmd "modinfo qla2xxx" "${hbaInfoFile}"
    [ `lsmod 2>/dev/null|grep -c "qla4xxx"` -ne 0 ] && exec_cmd "modinfo qla4xxx" "${hbaInfoFile}"
    [ `lsmod 2>/dev/null|grep -c "lpfc"` -ne 0 ] && exec_cmd "modinfo lpfc" "${hbaInfoFile}"
    [ `lsmod 2>/dev/null|grep -c "bfa"` -ne 0 ] && exec_cmd "modinfo bfa" "${hbaInfoFile}"
    [ `lsmod 2>/dev/null|grep -c "celerityfc"` -ne 0 ] && exec_cmd "modinfo celerityfc" "${hbaInfoFile}"
    [ `lsmod 2>/dev/null|grep -c "celerity8fc"` -ne 0 ] && exec_cmd "modinfo celerity8fc" "${hbaInfoFile}"
    [ `lsmod 2>/dev/null|grep -c "mlx4_ib"` -ne 0 ] && exec_cmd "modinfo mlx4_ib" "${hbaInfoFile}"
    [ `lsmod 2>/dev/null|grep -c "mlx4_ib"` -ne 0 ] && exec_cmd "ibstat" "${hbaInfoFile}"

    [ -e /sys/module/qla2xxx/parameters/ql2xfailover ] && exec_cmd "cat /sys/module/qla2xxx/parameters/ql2xfailover" "${hbaInfoFile}"
    [ -e /sys/module/qla2xxx/parameters/qlport_down_retry ] && exec_cmd "cat /sys/module/qla2xxx/parameters/qlport_down_retry" "${hbaInfoFile}"
    [ -e /sys/module/lpfc/parameters/lpfc_nodev_tmo ] && exec_cmd "cat /sys/module/lpfc/parameters/lpfc_nodev_tmo" "${hbaInfoFile}"

    [ -d /proc/scsi/qla2xxx/ ] && exec_cmd "cat /proc/scsi/qla2xxx/*" "${hbaInfoFile}"

    exec_cmd "cat /proc/scsi/scsi" "${hbaInfoFile}"
    exec_cmd "ls /proc/scsi/*" "${hbaInfoFile}"
    exec_cmd "lspci" "${hbaInfoFile}"
    exec_cmd "lspci -n" "${hbaInfoFile}"
    exec_cmd "lspci -v" "${hbaInfoFile}"
    exec_cmd "lspci -vv" "${hbaInfoFile}"

    return 0
}

function get_path_info()
{
    local up_version
    local v_version
    local r_version
    
    which upadmin >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        up_version=`upadmin show version | sed -n '1p' | awk -F ': ' '{print $2}'`
        v_version=`echo "${up_version}" | awk -F '.' '{print $1}'`
        r_version=`echo "${up_version}" | awk -F '.' '{print $2}'`
    fi
    
    if [ `which rpm 2>/dev/null|grep -c "/rpm$"` -ne 0 ]
    then
        exec_cmd "rpm -qa | grep -E \"UltraPath|[Uu][Pp][Ff]orLinux\"" "${upInfoFile}"
        exec_cmd "rpm -qi UltraPath" "${upInfoFile}"
        exec_cmd "rpm -qi [Uu][Pp][Ff]orLinux" "${upInfoFile}"
    fi

    if [ `which dpkg 2>/dev/null|grep -c "/dpkg$"` -ne 0 ]
    then
        exec_cmd "dpkg -s ultrapath" "${upInfoFile}"
    fi

    if [ -d /sys/class/scsi_host ]
    then
        print_event "ls /sys/class/scsi_host/*/state" "${upInfoFile}"

        for tmpFile in `ls /sys/class/scsi_host/*/state 2>/dev/null`
        do
            echo "${tmpFile}"
            echo -n "state:"
            cat "${tmpFile}"
        done >>"${upInfoFile}"
    fi

    if [ -f /usr/sbin/upTools ]
    then
        exec_cmd "upTools -S" "${upInfoFile}"
        exec_cmd "upTools -V" "${upInfoFile}"
        exec_cmd "upTools -c" "${upInfoFile}"
        exec_cmd "upTools -a" "${upInfoFile}"
    fi

    if [ `which upadmin 2>/dev/null|grep -c "/upadmin$"` -ne 0 ]
    then
        exec_cmd "upadmin show version" "${upInfoFile}"
        exec_cmd "upadmin show array" "${upInfoFile}"
        exec_cmd "upadmin show path" "${upInfoFile}"
        exec_cmd "upadmin show vlun" "${upInfoFile}"
        exec_cmd "upadmin show upconfig" "${upInfoFile}"
        upadmin show vlun | grep [0-9][0-9].* | awk '{print $1}' > .vlunid
        INDEX=1
        ID=`sed -n ${INDEX}p .vlunid`
        while [ "$ID" != "" ];
        do
            if [ $ID -le 16383 ]
            then
                exec_cmd "upadmin show vlun id=$ID" "${upInfoFile}"
                INDEX=`expr $INDEX + 1`
                ID=`sed -n ${INDEX}p .vlunid`
            fi
        done
        rm .vlunid

        if [ "`echo "${v_version}"`" = "8" ] && [ "`echo "${r_version}"`" = "03" ]
        then
            exec_cmd "upadmin show vlun type=all" "${upInfoFile}"            
            exec_cmd "upadmin show alarmenable" "${upInfoFile}"
            exec_cmd "upadmin show path_reliability_enable" "${upInfoFile}"
            upadmin show vlun type=all | grep [0-9][0-9].* | awk '{print $1}' > .vlunid
            INDEX=1
            ID=`sed -n ${INDEX}p .vlunid`
            while [ "$ID" != "" ];
            do
                if [ $ID -le 16383 ]
                then
                    exec_cmd "upadmin show vlun id=$ID type=all" "${upInfoFile}"  
                    INDEX=`expr $INDEX + 1`
                    ID=`sed -n ${INDEX}p .vlunid`
                fi
            done
            rm .vlunid
        fi
        exec_cmd "upadmin check status" "${upInfoFile}"
    fi

    if [ `which upadm 2>/dev/null|grep -c "/upadm$"` -ne 0 ]
    then
        exec_cmd "upadm show version" "${upInfoFile}"
        exec_cmd "upadm show path" "${upInfoFile}"
        exec_cmd "upadm show option" "${upInfoFile}"
        exec_cmd "upadm show array" "${upInfoFile}"
    fi

    return 0
}

function get_disk_info()
{
    [ -f /etc/fstab ] && exec_cmd "cat /etc/fstab" "${upInfoFile}"

    exec_cmd "df -a" "${upInfoFile}"
    exec_cmd "fdisk -l" "${upInfoFile}"
    exec_cmd "mount" "${upInfoFile}"
    exec_cmd "df -k" "${upInfoFile}"
    exec_cmd "df -Th" "${upInfoFile}"
    exec_cmd "df -kT" "${upInfoFile}"
    exec_cmd "ls -al /dev/nxup" "${upInfoFile}"
    exec_cmd "ls -al /dev/sd*" "${upInfoFile}"

    if [ `which lsscsi  2>/dev/null|grep -c "/lsscsi" ` -ne 0 ]
    then
        exec_cmd "lsscsi" "${upInfoFile}"
        exec_cmd "lsscsi -g" "${upInfoFile}"
        exec_cmd "lsscsi -H -v -d -g" "${upInfoFile}"
        exec_cmd "lsscsi -c -l -k" "${upInfoFile}"
        exec_cmd "lsscsi -v -d -g" "${upInfoFile}"
    fi

    exec_cmd "cat /proc/partitions" "${upInfoFile}"
    exec_cmd "ls -l /dev/disk/by-id" "${upInfoFile}"
    exec_cmd "ls -l /dev/disk/by-path" "${upInfoFile}"
    exec_cmd "ls -l /dev/disk/by-uuid" "${upInfoFile}"
    exec_cmd "ls -l /dev/disk/by-label" "${upInfoFile}"

    [ -d /sys/bus/scsi/drivers/sd/ ] && exec_cmd "ls -l /sys/bus/scsi/drivers/sd/" "${upInfoFile}"
    [ -d /sys/bus/scsi/drivers/NXUP/ ] && exec_cmd "ls -l /sys/bus/scsi/drivers/NXUP/" "${upInfoFile}"

    return 0
}

function cpKernelLog()
{
    local loop=0
    pwdPath=`pwd`

    if [ -d ${resultPath}/opt/UltraPath/log/kernel_log ]
    then
        cd ${resultPath}/opt/UltraPath/log/kernel_log
        rm -rf *
    fi

    if [ ! -d /opt/UltraPath/log/kernel_log ]
    then
        return 1
    fi

    cd /opt/UltraPath/log/kernel_log
    line=`ls -lt |awk '{print $9}'|grep xmp_log.tar.gz|wc -l`
    if [ $line -lt 0 ] || [ $line -eq 0 ]
    then
        return 1
    fi

    if [ $line -eq 1 ]
    then
        tar_pack1=`ls -lt |awk '{print $9}'|grep xmp_log.tar.gz|sed -n 1p`
        cp /opt/UltraPath/log/kernel_log/$tar_pack1 ${resultPath}/opt/UltraPath/log/kernel_log/
        cd "${pwdPath}"
        return 0
    fi

    tar_pack1=`ls -lt |awk '{print $9}'|grep xmp_log.tar.gz|sed -n 1p`
    tar_pack2=`ls -lt |awk '{print $9}'|grep xmp_log.tar.gz|sed -n 2p`
    cp /opt/UltraPath/log/kernel_log/$tar_pack1 ${resultPath}/opt/UltraPath/log/kernel_log/
    cp /opt/UltraPath/log/kernel_log/$tar_pack2 ${resultPath}/opt/UltraPath/log/kernel_log/

    if [ $line -gt 1 ]
    then
        print_yesOrNo "The dump log is too large, Do you want to save them?"
        if [ $? -ne 0 ]
        then
            return 0
        fi

        mkdir ${resultPath}/extralog
        for item in `ls -t | grep xmp_log.tar.gz`
        do
            if [ ${loop} -lt 2 ]; then
                let loop=loop+1
                continue
            fi
            cp $item ${resultPath}/extralog/
        done
    fi

    cd "${pwdPath}"
    return 0
}

cpOtherLogToExtraLog()
{
    pwdPath=`pwd`
    if [ ! -d ${resultPath}/extralog ]
    then
        mkdir ${resultPath}/extralog
    fi
    if [ -d ${resultPath}/opt/UltraPath/log/alarm_log ]
    then
        rm -rf ${resultPath}/opt/UltraPath/log/alarm_log
        cp -rf /opt/UltraPath/log/alarm_log/* ${resultPath}/extralog/ 2>/dev/null
    fi
    if [ -d ${resultPath}/opt/UltraPath/log/uil_log ]
    then
        rm -rf ${resultPath}/opt/UltraPath/log/uil_log
        cp -rf /opt/UltraPath/log/uil_log/* ${resultPath}/extralog/ 2>/dev/null
    fi
    if [ -d ${resultPath}/opt/UltraPath/log/install_log ]
    then
        rm -rf ${resultPath}/opt/UltraPath/log/install_log
        cp -rf /opt/UltraPath/log/install_log/* ${resultPath}/extralog/ 2>/dev/null
    fi
    cd "${pwdPath}"
    return 0
}

function get_log_info()
{
    local tmpFile=""
    local fileSize=0

    mkdir -p "${resultPath}/var/log"
    mkdir -p "${resultPath}/opt/UltraPath/log"

    if [ -f /var/log/messages ]
    then
        tail -n 20000 "/var/log/messages" > "${resultPath}/var/log/messages"

        if [ -f /var/log/messages.1 ]
        then
            copy_file "/var/log/messages.1"  "${resultPath}/var/log/"
        fi

        tmpFile=`ls /var/log/messages-* 2>/dev/null |tail -n 1`
        [ ! -z "${tmpFile}" ] && copy_file "${tmpFile}" "${resultPath}/var/log/"
    fi

    if [ -d /var/log/UltraPath ]
    then
        cp -rf /var/log/UltraPath/* "${resultPath}/opt/UltraPath/log"
    fi

    copy_file "/opt/UltraPath/log" "${resultPath}/opt/UltraPath/"
    if [ -d ${resultPath}/opt/UltraPath/log/kernel_log ]
    then
        curPath=`pwd`
        cd ${resultPath}/opt/UltraPath/log/kernel_log
        ls | grep -v "`ls |tail -1`" | xargs rm -rf 2>/dev/null
        cd ${curPath}
        exec_cmd "ls -l /opt/UltraPath/log/kernel_log" "${upInfoFile}"
    fi

    if [ -d ${resultPath}/opt/UltraPath/t80s_uninstallBak ]
    then
        rm -rf ${resultPath}/opt/UltraPath/t80s_uninstallBak
    fi

    cpKernelLog
    cpOtherLogToExtraLog
    copy_file "/opt/UltraPath/backup" "${resultPath}/opt/UltraPath/"
    copy_file "/var/log/UltraPath_Install.log" "${resultPath}/var/log/"

    copy_file "/var/log/boot.msg" "${resultPath}/var/log/"
    copy_file "/var/log/boot.omsg" "${resultPath}/var/log/"
    copy_file "/var/log/dmesg" "${resultPath}/var/log/"
    copy_file "/var/log/kernel" "${resultPath}/var/log/"
    copy_file "/var/log/boot.log" "${resultPath}/var/log/"
    copy_file /var/log/ultrapathhotadd "${resultPath}/var/log/"

    dmesg > "${dmesgFile}"

    return 0
}

function get_bootloader_info()
{
    mkdir -p "${resultPath}/boot/grub"
    mkdir -p "${resultPath}/boot/efi"

    copy_file "/boot/grub" "${resultPath}/boot/"
    copy_file "/etc/elilo.conf" "${resultPath}/elilo.conf"
    copy_file "/etc/lilo.conf" "${resultPath}/lilo.conf"
    copy_file "/boot/efi" "${resultPath}/boot/"
    copy_file "/boot/grub2" "${resultPath}/boot/"

    copy_file "/boot/mpp-`uname -r`.img" "${resultPath}/boot/"
    if [ -f /boot/nxup-*.img ] && [ `ls -l /boot/nxup-*.img | grep -c ">"` -eq 0 ]
    then
    copy_file "/boot/nxup-`uname -r`.img" "${resultPath}/boot/"
    fi
    copy_file "/boot/huaweistorage-`uname -r`.img" "${resultPath}/boot/"
    copy_file "/boot/initrd-`uname -r`" "${resultPath}/boot/"
    copy_file "/boot/System.map-`uname -r`" "${resultPath}/boot/"
    copy_file "/boot/ultrapath-`uname -r`" "${resultPath}/boot/"

    return 0
}

function get_srv_info()
{
    mkdir -p "${resultPath}/etc/init.d"

    [ -d /etc/init.d ] && exec_cmd "ls -l /etc/init.d" "${srvInfoFile}"
    [ -d /etc/rc3.d ] && exec_cmd "ls -l /etc/rc3.d" "${srvInfoFile}"
    [ -d /etc/rc5.d ] && exec_cmd "ls -l /etc/rc5.d" "${srvInfoFile}"
    [ -d /etc/rc6.d ] && exec_cmd "ls -l /etc/rc6.d" "${srvInfoFile}"
    [ -d /etc/rcS.d ] && exec_cmd "ls -l /etc/rcS.d" "${srvInfoFile}"
    [ -d /etc/init.d/rc3.d ] && exec_cmd "ls -l /etc/init.d/rc3.d" "${srvInfoFile}"
    [ -d /etc/init.d/rc5.d ] && exec_cmd "ls -l /etc/init.d/rc5.d" "${srvInfoFile}"
    [ -d /etc/init.d/rc6.d ] && exec_cmd "ls -l /etc/init.d/rc6.d" "${srvInfoFile}"
    [ -d /etc/init.d/boot.d ] && exec_cmd "ls -l /etc/init.d/boot.d" "${srvInfoFile}"

    curPath=`pwd`
    cd /etc/init.d/
    tar -zcf "${resultPath}/etc/init.d/init.d.tar.gz" "./" 2>/dev/null
    cd ${curPath}

    exec_cmd "chkconfig --list" "${srvInfoFile}"

    if [ `cat /etc/issue | grep -c "SUSE Linux Enterprise Server 12"` -ne 0 ]
    then
        exec_cmd "systemctl status nxup" "${srvInfoFile}"
    fi
    
    return 0
}

function get_iscsi_info()
{
    local iscsiIPs
    local iscsiIP

    if [ `which iscsiadm 2>/dev/null|grep -c "/iscsiadm$"` -ne 0 ]
    then
        exec_cmd "iscsiadm -m session" "${iscsiInfoFile}"
        exec_cmd "iscsiadm -m node" "${iscsiInfoFile}"
        exec_cmd "iscsiadm -V" "${iscsiInfoFile}"

        iscsiIPs=`iscsiadm -m node 2>/dev/null |awk -F ":" '{print $1}'`
        for iscsiIP in ${iscsiIPs}
        do
           exec_cmd "iscsiadm -m node -p ${iscsiIP}" "${iscsiInfoFile}"
        done
    fi

    if [ `which iscsid 2>/dev/null|grep -c "/iscsid$"` -ne 0 ]
    then
        exec_cmd "iscsid -v" "${iscsiInfoFile}"
    fi

    if [ `which iscsi-ls 2>/dev/null|grep -c "/iscsi-ls$"` -ne 0 ]
    then
        exec_cmd "iscsi-ls -c" "${iscsiInfoFile}"
    fi

    exec_cmd "modinfo iscsi" "${iscsiInfoFile}"

    mkdir -p "${resultPath}/etc/iscsi/"

    copy_file "/etc/iscsi.conf" "${resultPath}/etc/"
    copy_file "/etc/iscsi/iscsid.conf" "${resultPath}/etc/iscsi/"
    copy_file "/etc/iscsi/initiatorname.iscsi" "${resultPath}/etc/iscsi/"

    if [ -f /etc/init.d/iscsi ]
    then
        exec_cmd "/etc/init.d/iscsi status" "${iscsiInfoFile}"
    fi
    if [ -f /etc/init.d/open-iscsi ]
    then
        exec_cmd "/etc/init.d/open-iscsi status" "${iscsiInfoFile}"
    fi

    return 0
}

function get_sys_info()
{
    [ `which systool  2>/dev/null|grep -c "/systool"` -eq 0 ] && return 1

    local pwdPath=`pwd`

    cd /sys/class

    [ -d fc_host ] && exec_cmd "systool -c fc_host -v" "${hbaInfoFile}"
    [ -d iscsi_host ] && exec_cmd "systool -c iscsi_host -v" "${iscsiInfoFile}"
    [ -d fc_remote_ports ] && exec_cmd "systool -c fc_remote_ports -v" "${hbaInfoFile}"
    [ -d sas_host ] && exec_cmd "systool -c sas_host -v" "${hbaInfoFile}"
    [ -d iscsi_session ] && exec_cmd "systool -c iscsi_session -v" "${iscsiInfoFile}"
    [ -d iscsi_connection ] && exec_cmd "systool -c iscsi_connection -v" "${iscsiInfoFile}"
    [ -d scsi_host ] && exec_cmd "systool -c scsi_host -v" "${scsiInfoFile}"
    [ -d fc_transport ] && exec_cmd "systool -c fc_transport Cv" "${hbaInfoFile}"
    [ -d qla2xxx ] && exec_cmd "systool -m qla2xxx Cv" "${hbaInfoFile}"
    [ -d lpfc ] && exec_cmd "systool -m lpfc Cv" "${hbaInfoFile}"
    [ -d scsi_disk ] && exec_cmd "systool -c scsi_disk Cv" "${scsiInfoFile}"
    [ -d scsi ] && exec_cmd "systool -b scsi Cv" "${scsiInfoFile}"

    cd "${pwdPath}"

    return 0
}

function get_network_info()
{
    local tmpEth

    for tmpEth in `cat /proc/net/dev | awk '{if($2>0 && NR > 2) print substr($1, 0, index($1, ":"))}' | awk -F ":" '{print $1}'`
    do
        print_event "ethtool ${tmpEth}" "${networkFile}"
        ethtool ${tmpEth} 2>/dev/null
    done >> "${iscsiInfoFile}"

    return 0
}

function get_dm_info()
{
    if [ `which dmsetup  2>/dev/null|grep -c "/dmsetup" ` -ne 0 ]
    then
        exec_cmd "dmsetup ls" "${dmFile}"
        exec_cmd "dmsetup info" "${dmFile}"
        exec_cmd "dmsetup table" "${dmFile}"
    fi

    exec_cmd "multipath -ll" "${dmFile}"
    exec_cmd "ls /dev/mapper/" "${dmFile}"
    copy_file "/etc/multipath.conf" "${resultPath}/etc/"

    return 0
}

function get_lvm_info()
{
    mkdir -p "${resultPath}/etc/lvm/"

    exec_cmd "lvm version" "${lvmFile}"
    exec_cmd "pvscan -v" "${lvmFile}"
    exec_cmd "lvmdiskscan" "${lvmFile}"
    exec_cmd "vgdisplay -v" "${lvmFile}"
    exec_cmd "vgs -v" "${lvmFile}"
    exec_cmd "lvs -v" "${lvmFile}"
    exec_cmd "lvdisplay -v" "${lvmFile}"
    exec_cmd "lvm dumpconfig" "${lvmFile}"

    copy_file "/etc/lvm/lvm.conf" "${resultPath}/etc/lvm/"

    return 0
}

function get_udev_info()
{
    copy_file "/etc/udev" "${resultPath}/etc/"

    exec_cmd "udevinfo -e" "${udevFile}"
    exec_cmd "udevinfo -d" "${udevFile}"

    return 0
}

function get_dump_info()
{
    if [ ! -d ${outdir}/dump ]
    then
        mkdir -p ${outdir}/dump
    fi

    cp -rf  /var/crash/* "${outdir}/dump/" >/dev/null 2>&1

    return 0
}

function get_info()
{
    get_history_info
    get_os_info
    get_hba_info
    get_path_info
    get_disk_info

    if [ ${is_full} = "1" ]
    then
        get_log_info
    fi

    get_bootloader_info
    get_srv_info
    get_iscsi_info
    get_sys_info
    get_network_info
    get_dm_info
    get_lvm_info
    get_udev_info

    if [ ${is_collectdump} -eq 1 ] || [ ${is_clientdump} -eq 1 ]
    then
        get_dump_info
    fi

    return 0
}

function tar_result()
{
    local pwdPath=`pwd`

    cd "${hostInfoPath}"
    echo "begin compress files."

    sync
    sleep 2

    hostinfo_tarName=${outdir}/Linux-Info.tar.gz

    if [ ${is_collectdump} -eq 1 ] || [ ${is_clientdump} -eq 1 ]
    then
        mkdir -p ${outdir}/result
        cd ${outdir}/result
        cp -rf ${resultPath}/* ${outdir}/result/ 2>/dev/null
        mv ${outdir}/dump ${outdir}/result/ 2>/dev/null
        tar -zcf "${hostinfo_tarName}" * 2>/dev/null
    else
        cd "${resultPath}"
        tar -zcf "${hostinfo_tarName}" * 2>/dev/null
    fi

    if [ $? -ne 0 ]
    then
        print_error "compress files failed."
        host_exit 1
    else
        rm -rf "${resultPath}"
        rm -rf "${outdir}/result"
        echo "collect HostInfo successfully.the HostInfo is in [${hostinfo_tarName}]"
    fi

    cd "${pwdPath}"

    return 0
}

function check_disk_space()
{
    local spaceStr=""
    local freeSpace=""

    if [ -z "$1" ] || [ -z "$2" ]
    then
        print_error "param error[$1,$2]"
        host_exit 1
    fi

    spaceStr=`df -k "$1" 2>/dev/null | sed -n '$p' |  grep ".*[[:blank:]][[:blank:]]*[0-9][0-9]*[[:blank:]][[:blank:]]*[0-9][0-9]*%[[:blank:]][[:blank:]]*/.*$"`
    if [ "${spaceStr}" = "" ]
    then
        print_error "get disk free space failed.[$1]"
        host_exit 1
    fi

    freeSpace=`df -k "$1" 2>/dev/null | sed -n '$p' | grep ".*[[:blank:]][[:blank:]]*[0-9][0-9]*[[:blank:]][[:blank:]]*[0-9][0-9]*%[[:blank:]][[:blank:]]*/.*$" | awk '{print $(NF-2)}'`
    if [ -z "${freeSpace}" ] || [ ${freeSpace} -lt $2 ]
    then
        print_error  "more disk free space need[$1,$2k]"
        host_exit ${NO_ENOUGH_SPACE_EXITCODE}
    fi

    return 0
}

function print_yesOrNo()
{
    local userInput=""

    if [ ${is_quiet} = "1" ]
    then
        return 0
    fi

    echo "$1"

    while [ 1 -eq 1 ]
    do
        echo -n "<Y|N>:"

        read userInput
        userInput=$(echo "${userInput}" | tr '[A-Z]' '[a-z]')
        if [ "${userInput}" != "yes" ] && \
            [ "${userInput}" != "y" ] && \
            [ "${userInput}" != "no" ] && \
            [ "${userInput}" != "n" ]
        then
            echo "you select is incorrect[${userInput}]"
            continue
        fi

        if [ "${userInput}" = "yes" ] || \
                [ "${userInput}" = "y" ]
        then
            return 0
        else
            return 1
        fi
    done

    return 0
}


function notes_info()
{
    echo  "NOTE: the tool will collect hostInfo as following:
    1) linux history shell commands
    2) OS type, kernel type, system bit and system name
    3) HBA cards information, such as driver version, host ports and vendor information
    4) disks list
    5) hardware information, such as cpu, memory, pci and dev devices information
    6) log information
    7) UltraPath software information
    8) some important files"

    if [ ${is_collectdump} -eq 1 ] || [ ${is_clientdump} -eq 1 ]
    then
        echo "    9) dump file"
    fi

    if [ ${is_quiet} = "0" ]
    then
        print_yesOrNo "Do you want to collect these information?"
        if [ $? -ne 0 ]
        then
            rm -rf "${resultPath}"
            host_exit 2
        fi
    fi

    return 0
}

function checkfreespace()
{
    if [ -z "$1" ]
    then
        print_error "param error[$1]"
        host_exit 1
    fi

    if [ -d /var/crash ]
    then
        dumpfilesize=`du -s /var/crash |awk '{print $1}'`
    fi

    outfreespace=`df -k "$1" 2>/dev/null | sed -n '$p' | grep ".*[[:blank:]][[:blank:]]*[0-9][0-9]*[[:blank:]][[:blank:]]*[0-9][0-9]*%[[:blank:]][[:blank:]]*/.*$" | awk '{print $(NF-2)}'`
    allneedspac=`expr $dumpfilesize + 30000`

    if [ -z "${outfreespace}" ] || [ ${outfreespace} -lt ${allneedspac} ]
    then
        rm -rf ${resultPath}
        print_error  "more disk free space need[$1,${allneedspac}k]"
        host_exit ${NO_ENOUGH_SPACE_EXITCODE}
    fi
}

function getcollectdumpdir()
{
    if [ ${is_collectdump} = "0" ]
    then
        return 1
    fi
    
    echo "Please input the directory for the collected dump file. The space is large enough to save the dump informations, default is the current dir. "
    echo -n "Please input the directory : "
    read inputdirectory
    if [ "${inputdirectory}" = "" ]
    then
        dumpoutdir=${hostInfoPath}/
        if [ ! -d "${dumpoutdir}" ]
        then
            mkdir -p "${dumpoutdir}/"
        fi
    else
        if [ ! -d "${inputdirectory}" ]
        then
            echo "Current directory is not exist,please input a valid directory."
            host_exit ${FILE_DIR_NOT_EXIST}
        fi
        
        dumpoutdir="${inputdirectory}"
    fi
 
    outdir=${dumpoutdir}

    return 0
}

function main()
{
    if [ ${is_quiet} = "0" ]
    then
        print_yesOrNo "WARNING: Do you want to collect dump file? If yes,the dump file may be involved in the collection of private data or suspected by customer,please confirm with the customer before the collection."
        if [ $? -eq 0 ]
        then
            is_collectdump=1
            getcollectdumpdir
        else
            outdir=${hostInfoPath}/
        fi
    else
        outdir=${hostInfoPath}/
    fi

    if [ $is_collectdump -eq 1 ] || [ $is_clientdump -eq 1 ]
    then
        checkfreespace "${outdir}"
    else
        check_disk_space "${outdir}" 30000
    fi

    notes_info

    mkdir -p "${resultPath}"

    get_info
    tar_result

    return 0
}

CheckSystemType
lockShell "${lockFile}"

while [ $# -gt 0 ]
do
    param="$1"
    if [ $(echo "${param}"|grep -c "^-[^-]") -eq 1 ]
    then
        paramtmp=${param#-}
        paramLen=${#paramtmp}

        i=1
        for (( i=0; i<${paramLen}; i=i+1))
        do
            case "${paramtmp:${i}:1}" in
                "s")
                    is_quiet=1
                    ;;
                "f")
                    set_full=1
                    ;;
                "d")
                    is_clientdump=1
                    ;;
                *)
                    echo "Usage Error.[${param}]"
                    useage
                    ;;
            esac
        done
    elif [ $(echo "${param}"|grep -c "^--[^-]") -eq 1 ]
    then
        paramtmp=${param#--}
        case "${paramtmp}" in
            *)
                echo "Usage Error.[${param}]"
                useage
                ;;
        esac
    else
        echo "Usage Error.[${param}]"
        useage
    fi
    shift
done

if [ ${is_quiet} = "1" ]
then
    if [ ${set_full} = "0" ]
    then
        is_full=0
    fi
fi

main
host_exit 0
